@@ -17,11 +17,12 @@ from .utils import *
 from .version import *
 from .workchains import *
 
-__all__ = (events.__all__ + exceptions.__all__ + processes.__all__ +
-           utils.__all__ + futures.__all__ + mixins.__all__ +
-           persistence.__all__ + communications.__all__ + process_comms.__all__ +
-           version.__all__, process_listener.__all__ + workchains.__all__ + loaders.__all__ +
-           ports.__all__ + process_states.__all__)
+__all__ = (
+    events.__all__ + exceptions.__all__ + processes.__all__ + utils.__all__ +
+    futures.__all__ + mixins.__all__ + persistence.__all__ +
+    communications.__all__ + process_comms.__all__ + version.__all__,
+    process_listener.__all__ + workchains.__all__ + loaders.__all__ +
+    ports.__all__ + process_states.__all__)
 
 
 # Do this se we don't get the "No handlers could be found..." warnings that will be produced
