@@ -98,4 +98,22 @@ class TestPortNamespace(TestCase):
         port_namespace_sub[self.BASE_PORT_NAME] = self.port
 
         with self.assertRaises(ValueError):
-            self.port_namespace.create_port_namespace('sub.nested.space.' + self.BASE_PORT_NAME + '.further')
\ No newline at end of file
+            self.port_namespace.create_port_namespace('sub.nested.space.' + self.BASE_PORT_NAME + '.further')
+
+    def test_port_namespace_set_valid_type(self):
+        """
+        Setting a valid type for a PortNamespace should automatically mark it as dynamic. Conversely, setting
+        the valid_type equal to None should revert dynamic to False
+        """
+        self.assertEquals(self.port_namespace.dynamic, False)
+        self.assertEquals(self.port_namespace.valid_type, None)
+
+        self.port_namespace.valid_type = int
+
+        self.assertEquals(self.port_namespace.dynamic, True)
+        self.assertEquals(self.port_namespace.valid_type, int)
+
+        self.port_namespace.valid_type = None
+
+        self.assertEquals(self.port_namespace.dynamic, False)
+        self.assertEquals(self.port_namespace.valid_type, None)
